==> asterisk-spandsp-plugins_0.0.20050203.orig.tar.gz: asterisk-spandsp-plugins-0.0.20050203/app_txfax_test.c <==
/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Trivial application to send a TIFF file as a FAX
 * 
 * Copyright (C) 2003, Steve Underwood
 *
 * Steve Underwood <steveu@coppice.org>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <pthread.h>
#include <errno.h>
#include <tiffio.h>

#include <spandsp.h>

static char *tdesc = "Trivial FAX Transmit Application";

static char *app = "TxFAX";

static char *synopsis = "Send a FAX file";

static char *descrip = 
"  TxFAX(filename[|option]):  Send a given file name as a FAX. Returns -1\n"
"Uses LOCALSTATIONID to identify itself to the remote end.\n"
"Sets REMOTESTATIONID to the receiver CSID.\n"
"Returns -1 when the user hangs up, or if the file does not exist.\n"
"Returns 0 otherwise.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

#define MAX_BLOCK_SIZE 240

static void t30_flush(t30_state_t *s, int which)
{
    //TODO:
}
/*- End of function --------------------------------------------------------*/

static void phase_e_handler(t30_state_t *s, void *user_data, int result)
{
    struct ast_channel *chan;
    char far_ident[21];
    
    chan = (struct ast_channel *) user_data;
    if (result)
    {
        fax_get_far_ident(s, far_ident);
        pbx_builtin_setvar_helper(chan, "REMOTESTATIONID", far_ident);
    }
    else
    {
        ast_log(LOG_DEBUG, "==============================================================================\n");
        ast_log(LOG_DEBUG, "Fax send not successful.\n");
        ast_log(LOG_DEBUG, "==============================================================================\n");
    }
}
/*- End of function --------------------------------------------------------*/

static void *fax_gen_alloc(struct ast_channel *chan, void *params)
{
    return params;
}
/*- End of function --------------------------------------------------------*/

static void fax_gen_release(struct ast_channel *chan, void *data)
{
    return;
}
/*- End of function --------------------------------------------------------*/

static int fax_gen_generate(struct ast_channel *chan, void *data, int len, int samples)
{
    struct ast_frame f;
    uint8_t buf[sizeof(uint16_t)*MAX_BLOCK_SIZE + 2*AST_FRIENDLY_OFFSET];
    t30_state_t *fax;

    fax = (t30_state_t *) data;
    if (len > sizeof (buf))
    {
        ast_log (LOG_WARNING, "Only doing %d bytes (%d bytes requested)\n", (int)(sizeof (buf) / sizeof (signed short)), len);
        len = sizeof (buf);
        samples = len / 2;
    }
    len = fax_tx_process(fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
    f.frametype = AST_FRAME_VOICE;
    f.subclass = AST_FORMAT_SLINEAR;
    f.offset = AST_FRIENDLY_OFFSET;
    f.mallocd = 0;
    f.data = &buf[AST_FRIENDLY_OFFSET];
    f.datalen = len*sizeof(int16_t);
    f.samples = len;
    f.src = "app_txfax";
    if (ast_write(chan, &f) < 0)
    {
        ast_log (LOG_WARNING, "Failed to write frame to '%s': %s\n", chan->name, strerror(errno));
        return -1;
    }
    return 0;
}
/*- End of function --------------------------------------------------------*/

static struct ast_generator fax_gen =
{
    alloc:fax_gen_alloc,
    release:fax_gen_release,
    generate:fax_gen_generate,
};

static int txfax_exec(struct ast_channel *chan, void *data)
{
    int res = 0;
    int count = 0;
    int percentflag = 0;
    char fil[256];
    char tmp[256];
    char *vdata;
    char *localident;
    int i;
    t30_state_t fax;
    int calling_party;
    
    struct localuser *u;
    struct ast_frame *inf = NULL;

    int original_read_fmt;
    int original_write_fmt;
    
    if (chan == NULL)
    {
        ast_log(LOG_WARNING, "Fax transmit channel is NULL. Giving up.\n");
        return -1;
    }
    vdata = data;

    /* The next few lines of code parse out the filename and header from the input string */
    if (!vdata)
    {
        /* No data implies no filename or anything is present */
        ast_log(LOG_WARNING, "Txfax requires an argument (filename)\n");
        return -1;
    }
    
    for (i = 0;  vdata[i]  &&  (vdata[i] != ':')  &&  (vdata[i] != '|');  i++)
    {
        if ((vdata[i] == '%')  &&  (vdata[i + 1] == 'd'))
            percentflag = 1;                      /* the wildcard is used */
        
        if (i == strlen(vdata))
        {
            ast_log(LOG_WARNING, "No extension found\n");
            return -1;
        }
        fil[i] = vdata[i];
    }
    fil[i] = '\0';
    calling_party = FALSE;
    if (vdata[i] == '|')
    {
        i++;
        if (strcmp(vdata + i, "caller") == 0)
            calling_party = TRUE;
    }
    /* Done parsing */

    /* These are to allow the use of the %d in the config file for a wild card of sort to
       create a new file with the inputed name scheme */
    if (percentflag)
    {
        do
        {
            snprintf(tmp, 256, fil, count);
            count++;
        }
        while (ast_fileexists(tmp, "", chan->language) != -1);
    }
    else
    {
        strncpy(tmp, fil, 256 - 1);
    }

    LOCAL_USER_ADD(u);

    if (chan->_state != AST_STATE_UP)
    {
        /* Shouldn't need this, but checking to see if channel is already answered
         * Theoretically asterisk should already have answered before running the app */
        res = ast_answer(chan);
    }
    
    if (!res)
    {
        original_read_fmt = chan->readformat;
        if (original_read_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_read_format(chan, AST_FORMAT_SLINEAR);
            if (res < 0)
            {
                ast_log(LOG_WARNING, "Unable to set to linear read mode, giving up\n");
                return -1;
            }
        }
        original_write_fmt = chan->writeformat;
        if (original_write_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_write_format(chan, AST_FORMAT_SLINEAR);
            if (res < 0)
            {
                ast_log(LOG_WARNING, "Unable to set to linear write mode, giving up\n");
                res = ast_set_read_format(chan, original_read_fmt);
                if (res)
                    ast_log(LOG_WARNING, "Unable to restore read format on '%s'\n", chan->name);
                return -1;
            }
        }
        fax_init(&fax, calling_party, NULL);

        localident = pbx_builtin_getvar_helper(chan, "LOCALSTATIONID");
        fax_set_local_ident(&fax, localident);
        fax_set_tx_file(&fax, tmp);
        //fax_set_phase_b_handler(&fax, phase_b_handler, chan);
        //fax_set_phase_d_handler(&fax, phase_d_handler, chan);
        fax_set_phase_e_handler(&fax, phase_e_handler, chan);
	if (ast_activate_generator(chan, &fax_gen, &fax) < 0)
        {
	    LOCAL_USER_REMOVE (u);
	    ast_log (LOG_ERROR, "Failed to activate generator on '%s'\n", chan->name);
	    return -1;
	}

        while (ast_waitfor(chan, -1) > -1)
        {
            inf = ast_read(chan);
            if (inf == NULL)
            {
                res = -1;
                break;
            }
            if (inf->frametype == AST_FRAME_VOICE)
            {
                if (fax_rx_process(&fax, inf->data, inf->samples))
                    break;
            }
            ast_frfree(inf);
        }
        if (inf == NULL)
        {
            ast_log(LOG_DEBUG, "Got hangup\n");
            res = -1;
        }
	ast_deactivate_generator(chan);
        if (original_read_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_read_format(chan, original_read_fmt);
            if (res)
                ast_log(LOG_WARNING, "Unable to restore read format on '%s'\n", chan->name);
        }
        if (original_write_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_write_format(chan, original_write_fmt);
            if (res)
                ast_log(LOG_WARNING, "Unable to restore write format on '%s'\n", chan->name);
        }
    }
    else
    {
        ast_log(LOG_WARNING, "Could not answer channel '%s'\n", chan->name);
    }
    LOCAL_USER_REMOVE(u);
    return res;
}
/*- End of function --------------------------------------------------------*/

int unload_module(void)
{
    STANDARD_HANGUP_LOCALUSERS;
    return ast_unregister_application(app);
}
/*- End of function --------------------------------------------------------*/

int load_module(void)
{
    return ast_register_application(app, txfax_exec, synopsis, descrip);
}
/*- End of function --------------------------------------------------------*/

char *description(void)
{
    return tdesc;
}
/*- End of function --------------------------------------------------------*/

int usecount(void)
{
    int res;

    STANDARD_USECOUNT(res);
    return res;
}
/*- End of function --------------------------------------------------------*/

char *key(void)
{
    return ASTERISK_GPL_KEY;
}
/*- End of function --------------------------------------------------------*/
/*- End of file ------------------------------------------------------------*/

==> asterisk-spandsp-plugins_0.0.20050227.orig.tar.gz: asterisk-spandsp-plugins-0.0.20050227/app_txfax_test.c <==
/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Trivial application to send a TIFF file as a FAX
 * 
 * Copyright (C) 2003, Steve Underwood
 *
 * Steve Underwood <steveu@coppice.org>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <pthread.h>
#include <errno.h>
#include <tiffio.h>

#include <spandsp.h>

static char *tdesc = "Trivial FAX Transmit Application";

static char *app = "TxFAX";

static char *synopsis = "Send a FAX file";

static char *descrip = 
"  TxFAX(filename[|caller]):  Send a given TIFF file to the channel as a FAX.\n"
"The \"caller\" option makes the application behave as a calling machine,\n"
"rather than the answering machine. The default behaviour is to behave as\n"
"an answering machine.\n"
"Uses LOCALSTATIONID to identify itself to the remote end.\n"
"Sets REMOTESTATIONID to the receiver CSID.\n"
"Returns -1 when the user hangs up, or if the file does not exist.\n"
"Returns 0 otherwise.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

#define MAX_BLOCK_SIZE 240

static void t30_flush(t30_state_t *s, int which)
{
    //TODO:
}
/*- End of function --------------------------------------------------------*/

static void phase_e_handler(t30_state_t *s, void *user_data, int result)
{
    struct ast_channel *chan;
    char far_ident[21];
    
    chan = (struct ast_channel *) user_data;
    if (result)
    {
        fax_get_far_ident(s, far_ident);
        pbx_builtin_setvar_helper(chan, "REMOTESTATIONID", far_ident);
    }
    else
    {
        ast_log(LOG_DEBUG, "==============================================================================\n");
        ast_log(LOG_DEBUG, "Fax send not successful.\n");
        ast_log(LOG_DEBUG, "==============================================================================\n");
    }
}
/*- End of function --------------------------------------------------------*/

static void *fax_gen_alloc(struct ast_channel *chan, void *params)
{
    return params;
}
/*- End of function --------------------------------------------------------*/

static void fax_gen_release(struct ast_channel *chan, void *data)
{
    return;
}
/*- End of function --------------------------------------------------------*/

static int fax_gen_generate(struct ast_channel *chan, void *data, int len, int samples)
{
    struct ast_frame f;
    uint8_t buf[sizeof(uint16_t)*MAX_BLOCK_SIZE + 2*AST_FRIENDLY_OFFSET];
    t30_state_t *fax;

    fax = (t30_state_t *) data;
    if (len > sizeof (buf))
    {
        ast_log (LOG_WARNING, "Only doing %d bytes (%d bytes requested)\n", (int)(sizeof (buf) / sizeof (signed short)), len);
        len = sizeof (buf);
        samples = len / 2;
    }
    len = fax_tx_process(fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
    f.frametype = AST_FRAME_VOICE;
    f.subclass = AST_FORMAT_SLINEAR;
    f.offset = AST_FRIENDLY_OFFSET;
    f.mallocd = 0;
    f.data = &buf[AST_FRIENDLY_OFFSET];
    f.datalen = len*sizeof(int16_t);
    f.samples = len;
    f.src = "app_txfax";
    if (ast_write(chan, &f) < 0)
    {
        ast_log (LOG_WARNING, "Failed to write frame to '%s': %s\n", chan->name, strerror(errno));
        return -1;
    }
    return 0;
}
/*- End of function --------------------------------------------------------*/

static struct ast_generator fax_gen =
{
    alloc:fax_gen_alloc,
    release:fax_gen_release,
    generate:fax_gen_generate,
};

static int txfax_exec(struct ast_channel *chan, void *data)
{
    int res = 0;
    int count = 0;
    int percentflag = 0;
    char fil[256];
    char tmp[256];
    char *vdata;
    char *localident;
    int i;
    t30_state_t fax;
    int calling_party;
    
    struct localuser *u;
    struct ast_frame *inf = NULL;

    int original_read_fmt;
    int original_write_fmt;
    
    if (chan == NULL)
    {
        ast_log(LOG_WARNING, "Fax transmit channel is NULL. Giving up.\n");
        return -1;
    }
    vdata = data;

    /* The next few lines of code parse out the filename and header from the input string */
    if (!vdata)
    {
        /* No data implies no filename or anything is present */
        ast_log(LOG_WARNING, "Txfax requires an argument (filename)\n");
        return -1;
    }
    
    for (i = 0;  vdata[i]  &&  (vdata[i] != ':')  &&  (vdata[i] != '|');  i++)
    {
        if ((vdata[i] == '%')  &&  (vdata[i + 1] == 'd'))
            percentflag = 1;                      /* the wildcard is used */
        
        if (i == strlen(vdata))
        {
            ast_log(LOG_WARNING, "No extension found\n");
            return -1;
        }
        fil[i] = vdata[i];
    }
    fil[i] = '\0';
    calling_party = FALSE;
    if (vdata[i] == '|')
    {
        i++;
        if (strcmp(vdata + i, "caller") == 0)
            calling_party = TRUE;
    }
    /* Done parsing */

    /* These are to allow the use of the %d in the config file for a wild card of sort to
       create a new file with the inputed name scheme */
    if (percentflag)
    {
        do
        {
            snprintf(tmp, 256, fil, count);
            count++;
        }
        while (ast_fileexists(tmp, "", chan->language) != -1);
    }
    else
    {
        strncpy(tmp, fil, 256 - 1);
    }

    LOCAL_USER_ADD(u);

    if (chan->_state != AST_STATE_UP)
    {
        /* Shouldn't need this, but checking to see if channel is already answered
         * Theoretically asterisk should already have answered before running the app */
        res = ast_answer(chan);
    }
    
    if (!res)
    {
        original_read_fmt = chan->readformat;
        if (original_read_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_read_format(chan, AST_FORMAT_SLINEAR);
            if (res < 0)
            {
                ast_log(LOG_WARNING, "Unable to set to linear read mode, giving up\n");
                return -1;
            }
        }
        original_write_fmt = chan->writeformat;
        if (original_write_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_write_format(chan, AST_FORMAT_SLINEAR);
            if (res < 0)
            {
                ast_log(LOG_WARNING, "Unable to set to linear write mode, giving up\n");
                res = ast_set_read_format(chan, original_read_fmt);
                if (res)
                    ast_log(LOG_WARNING, "Unable to restore read format on '%s'\n", chan->name);
                return -1;
            }
        }
        fax_init(&fax, calling_party, NULL);

        localident = pbx_builtin_getvar_helper(chan, "LOCALSTATIONID");
        fax_set_local_ident(&fax, localident);
        fax_set_tx_file(&fax, tmp);
        //fax_set_phase_b_handler(&fax, phase_b_handler, chan);
        //fax_set_phase_d_handler(&fax, phase_d_handler, chan);
        fax_set_phase_e_handler(&fax, phase_e_handler, chan);
	if (ast_activate_generator(chan, &fax_gen, &fax) < 0)
        {
	    LOCAL_USER_REMOVE (u);
	    ast_log (LOG_ERROR, "Failed to activate generator on '%s'\n", chan->name);
	    return -1;
	}

        while (ast_waitfor(chan, -1) > -1)
        {
            inf = ast_read(chan);
            if (inf == NULL)
            {
                res = -1;
                break;
            }
            if (inf->frametype == AST_FRAME_VOICE)
            {
                if (fax_rx_process(&fax, inf->data, inf->samples))
                    break;
            }
            ast_frfree(inf);
        }
        if (inf == NULL)
        {
            ast_log(LOG_DEBUG, "Got hangup\n");
            res = -1;
        }
	ast_deactivate_generator(chan);
        if (original_read_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_read_format(chan, original_read_fmt);
            if (res)
                ast_log(LOG_WARNING, "Unable to restore read format on '%s'\n", chan->name);
        }
        if (original_write_fmt != AST_FORMAT_SLINEAR)
        {
            res = ast_set_write_format(chan, original_write_fmt);
            if (res)
                ast_log(LOG_WARNING, "Unable to restore write format on '%s'\n", chan->name);
        }
    }
    else
    {
        ast_log(LOG_WARNING, "Could not answer channel '%s'\n", chan->name);
    }
    LOCAL_USER_REMOVE(u);
    return res;
}
/*- End of function --------------------------------------------------------*/

int unload_module(void)
{
    STANDARD_HANGUP_LOCALUSERS;
    return ast_unregister_application(app);
}
/*- End of function --------------------------------------------------------*/

int load_module(void)
{
    return ast_register_application(app, txfax_exec, synopsis, descrip);
}
/*- End of function --------------------------------------------------------*/

char *description(void)
{
    return tdesc;
}
/*- End of function --------------------------------------------------------*/

int usecount(void)
{
    int res;

    STANDARD_USECOUNT(res);
    return res;
}
/*- End of function --------------------------------------------------------*/

char *key(void)
{
    return ASTERISK_GPL_KEY;
}
/*- End of function --------------------------------------------------------*/
/*- End of file ------------------------------------------------------------*/
